package dataset

import (
	"strings"

	"code.gitea.io/gitea/models"
	"code.gitea.io/gitea/modules/auth"
	"code.gitea.io/gitea/modules/base"
	"code.gitea.io/gitea/modules/context"
	"code.gitea.io/gitea/modules/log"
	"code.gitea.io/gitea/modules/setting"
	"code.gitea.io/gitea/routers/repo"
	"github.com/unknwon/com"
)

const (
	tplDataSet base.TplName = "datasets/index"
	tplShow    base.TplName = "datasets/show"
	tplCreate  base.TplName = "datasets/create"
	tplEdit    base.TplName = "datasets/edit"
)

type ListOptions struct {
	PageSize int
	Page     int // start from 1
}

func MyList(ctx *context.Context) {

	var (
		datasets []*models.Dataset
		count    int64
		err      error
		orderBy  models.SearchOrderBy
	)

	page := ctx.QueryInt("page")
	if page <= 0 {
		page = 1
	}
	switch ctx.Query("sort") {
	case "newest":
		orderBy = models.SearchOrderByNewest
	case "oldest":
		orderBy = models.SearchOrderByOldest
	case "recentupdate":
		orderBy = models.SearchOrderByRecentUpdated
	case "leastupdate":
		orderBy = models.SearchOrderByLeastUpdated
	case "reversealphabetically":
		orderBy = models.SearchOrderByAlphabeticallyReverse
	case "alphabetically":
		orderBy = models.SearchOrderByAlphabetically
	case "reversesize":
		orderBy = models.SearchOrderBySizeReverse
	case "size":
		orderBy = models.SearchOrderBySize
	case "moststars":
		orderBy = models.SearchOrderByStarsReverse
	case "feweststars":
		orderBy = models.SearchOrderByStars
	case "mostforks":
		orderBy = models.SearchOrderByForksReverse
	case "fewestforks":
		orderBy = models.SearchOrderByForks
	default:
		ctx.Data["SortType"] = "recentupdate"
		orderBy = models.SearchOrderByRecentUpdated
	}
	keyword := strings.Trim(ctx.Query("q"), " ")
	var userID int64
	if ctx.User != nil {
		userID = ctx.User.ID
	}
	opts := &models.SearchDatasetOptions{
		Keyword:       keyword,
		IsPublic:      true,
		SearchOrderBy: orderBy,
		OwnerID:       userID,
		ListOptions: models.ListOptions{
			Page:     page,
			PageSize: setting.UI.ExplorePagingNum,
		},
	}

	if len(opts.SearchOrderBy) == 0 {
		opts.SearchOrderBy = models.SearchOrderByAlphabetically
	}

	datasets, count, err = models.SearchDataset(opts)
	if err != nil {
		ctx.ServerError("SearchDatasets", err)
		return
	}

	pager := context.NewPagination(int(count), opts.PageSize, page, 5)
	pager.SetDefaultParams(ctx)
	ctx.Data["Page"] = pager

	ctx.Data["datasets"] = datasets
	ctx.Data["datasetsCount"] = count
	log.Debug("[dataset] mylist...\n")
	ctx.HTML(200, tplDataSet)
}

func Create(ctx *context.Context) {
	ctx.HTML(200, tplCreate)
}

func CreatePost(ctx *context.Context, form auth.CreateDatasetForm) {
	ctx.Data["Title"] = ctx.Tr("new_dataset")

	ctxUser := ctx.User

	if ctx.HasError() {
		ctx.HTML(200, tplCreate)
		return
	}

	status := models.DatasetStatusPublic
	if form.Private {
		status = models.DatasetStatusPrivate
	}

	var err error
	opts := models.Dataset{
		Title:       form.Title,
		Category:    form.Category,
		Description: form.Description,
		License:     form.License,
		Task:        form.Task,
		ReleaseID:   form.ReleaseID,
		UserID:      ctxUser.ID,
		Status:      status,
	}

	if err = models.CreateDataset(&opts); err != nil {
		log.Error("%v", err)
	}

	var attachmentUUIDs []string
	if setting.Attachment.Enabled {
		attachmentUUIDs = form.Files
	}

	if err = models.AddDatasetAttachments(opts.ID, attachmentUUIDs); err != nil {
		log.Error("%v", err)
	}

	if err == nil {
		log.Trace("Dataset created [%d]: %s/%s", opts.ID, ctxUser.Name, opts.Title)
		ctx.Redirect(setting.AppSubURL + "/datasets/" + com.ToStr(opts.ID))
		return
	}
}

func Show(ctx *context.Context) {
	ctx.Data["Title"] = ctx.Tr("dataset.show_dataset")

	user := ctx.User
	dataset, err := models.GetOwnerDatasetByID(ctx.ParamsInt64((":id")), user)
	if err != nil {
		ctx.NotFound("GetDataset", err)
		return
	}

	err = models.GeDatasetAttachments(dataset)
	if err != nil {
		ctx.ServerError("GetDatasetAttachments", err)
		return
	}

	isOwner := dataset.UserID == user.ID

	ctx.Data["dataset"] = dataset
	ctx.Data["IsOwner"] = isOwner

	ctx.HTML(200, tplShow)
}

// EditDataset render release edit page
func EditDataset(ctx *context.Context) {
	ctx.Data["Title"] = ctx.Tr("dataset.edit_dataset")
	ctx.Data["PageIsEditDataset"] = true
	repo.RenderAttachmentSettings(ctx)
	rel, err := models.GetOwnerDatasetByID(ctx.ParamsInt64(":id"), ctx.User)
	if err != nil {
		ctx.NotFound("GetDataset", err)
		return
	}

	ctx.Data["ID"] = rel.ID
	ctx.Data["title"] = rel.Title
	ctx.Data["private"] = rel.IsPrivate()
	ctx.Data["description"] = rel.Description
	ctx.Data["category"] = rel.Category
	ctx.Data["task"] = rel.Task
	ctx.Data["license"] = rel.License

	ctx.HTML(200, tplEdit)
}

func EditDatasetPost(ctx *context.Context, form auth.EditDatasetForm) {
	ctx.Data["Title"] = ctx.Tr("dataset.edit_dataset")

	rel, err := models.GetDatasetByID(ctx.ParamsInt64(":id"))
	if err != nil {
		ctx.ServerError("GetDataset", err)
		return
	}
	ctx.Data["ID"] = rel.ID
	ctx.Data["title"] = rel.Title
	ctx.Data["description"] = rel.Description
	ctx.Data["category"] = rel.Category
	ctx.Data["task"] = rel.Task
	ctx.Data["license"] = rel.License
	ctx.Data["private"] = rel.IsPrivate()

	if ctx.HasError() {
		ctx.HTML(200, tplCreate)
		return
	}

	var attachmentUUIDs []string
	if setting.Attachment.Enabled {
		attachmentUUIDs = form.Files
	}

	status := models.DatasetStatusPublic
	if form.Private {
		status = models.DatasetStatusPrivate
	}

	rel.Title = form.Title
	rel.Status = status
	rel.Description = form.Description
	rel.Category = form.Category
	rel.Task = form.Task
	rel.License = form.License
	if err = models.UpdateDataset(models.DefaultDBContext(), rel); err != nil {
		log.Error("%v", err)
	}

	if err = models.AddDatasetAttachments(rel.ID, attachmentUUIDs); err != nil {
		log.Error("%v", err)
	}
	log.Trace("Dataset updated: %d", rel.ID)

	ctx.Redirect(setting.AppSubURL + "/datasets/" + com.ToStr(rel.ID))
}

func Delete(ctx *context.Context) {
	log.Debug("[dataset] Delete...\n")
}
